function [focalLength1 , focalLength2] = calcFocalLength( fMatrix , ...
                                                      focus_base, CalculatingMode)

vectorK = [0 ; 0 ; 1];

% 計算に必要なパラメータの計算
kFk    = dot(vectorK, fMatrix * vectorK);
Fk2    = norm(fMatrix * vectorK, 'fro')^2;
Ftk2   = norm(fMatrix' * vectorK, 'fro')^2;
kFFtFk = dot(vectorK, fMatrix * fMatrix' * fMatrix * vectorK);
FFtk2  = norm(fMatrix  * fMatrix' * vectorK, 'fro')^2;
FtFk2  = norm(fMatrix' * fMatrix  * vectorK, 'fro')^2;
FFt2   = norm(fMatrix * fMatrix', 'fro')^2;
F2     = norm(fMatrix, 'fro')^2;

% 平均焦点距離で計算
if strcmpi(CalculatingMode, 'free')

  % 注視か否かを判定
  if( Fk2 < Ftk2 )
    condition = 0.1 * Fk2 / focus_base;
  else
    condition = 0.1 * Ftk2 / focus_base;
  end

  if( abs(kFk) < condition )
    disp('Error : this is Gaze')
  end

  % エピ極点を求める
  [U, S, V] = svd( fMatrix * fMatrix');
  e1 = V(: , 3);
  e1 = e1 / norm(e1);
  
  [U, S, V] = svd( fMatrix' * fMatrix);
  e2 = V(: , 3);
  e2 = e2 / norm(e2);

  % Bougnoux の公式用の計算
  e1k    = norm(cross(e1, vectorK), 'fro')^2;
  e2k    = norm(cross(e2, vectorK), 'fro')^2;

  % 焦点距離の計算
  xi  = (Fk2  - (kFFtFk * e2k) / kFk) / (e2k * Ftk2 - kFk^2);
  eta = (Ftk2 - (kFFtFk * e1k) / kFk) / (e1k * Fk2  - kFk^2);

  % へシアンの計算
  H11 = (2 * kFk^4 * eta^2)    + (4 * kFk^2 * Ftk2 * eta) + (2 * Ftk2^2) - (kFk^2 * eta + Ftk2)^2;
  H22 = (2 * kFk^4 * xi^2)     + (4 * kFk^2 * Fk2 * xi)   + (2 * Fk2^2)  - (kFk^2 * xi + Fk2)^2;
  H12 = (4 * kFk^4 * xi * eta) + (4 * kFk^2 * (Ftk2 * xi + Fk2 * eta)) + (4 * kFk * kFFtFk)...
      - ((kFk^2 * xi + Fk2) * (kFk^2 * eta + Ftk2)) - kFk^2 * ((kFk^2 * xi * eta) + (Ftk2 * xi) + (Fk2 * eta) + F2);

  % 重み付き平均の計算
  xi  = ((H11 + H12) * xi + (H22 + H12) * eta) / (H11 + (2 * H12) + H22);
  eta = xi;

  % 焦点距離が虚数ならば強制終了
  if((1 + xi) <= 0)
    return
  end

  focalLength1 = focus_base / sqrt(1 + xi);
  focalLength2 = focus_base / sqrt(1 + eta);
end


% 固定焦点距離で計算
if strcmpi(CalculatingMode, 'fixed')
  
  a1 = 0.5 * kFk^4;
  a2 = kFk^2 * (Ftk2 + Fk2);
  a3 = 0.5 * ((Ftk2 - Fk2)^2) + kFk * ((4.0 * kFFtFk) - (kFk * F2));
  a4 = (2.0 * (FFtk2 + FtFk2)) - ((Ftk2 + Fk2) * F2);
  a5 = FFt2 - (0.5 * F2^2);

  % (k,Fk) = 0 かを判定
  if( Fk2 < Ftk2 )
    condition = 0.1 * Fk2 / focus_base;
  else
    condition = 0.1 * Ftk2 / focus_base;
  end

  if( abs(kFk) < condition )
    xi = -a4 / (2 * a3);
  else
    K = [4*a1 3*a2 2*a3 a4];
    r = roots(K);
    xi3 = r(1);
    xi1 = r(3);
    Kxi3 = a1*xi3^4 + a2*xi3^3 + a3*xi3^2 + a4*xi3 + a5;
    Kxi1 = a1*xi1^4 + a2*xi1^3 + a3*xi1^2 + a4*xi1 + a5;

    if((xi3 <= -1) || (Kxi3 < 0) || (Kxi1 <= Kxi3))
      xi = xi1;
    elseif((0 <= Kxi3) || (Kxi3 < Kxi1))
      xi = xi3;
    else
      xi = -1;
    end
  end

  % 焦点距離が虚数ならば強制終了
  if((1 + xi) <= 0)
    return
  end

  focalLength1 = focus_base / sqrt(1+xi);
  focalLength2 = focus_base / sqrt(1+xi);
end
